# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QLabel, QWidget, QVBoxLayout
class Label(QWidget):
    def __init__(self, parent, icon_path, move_icon_path, left = False):
        super(Label, self).__init__()
        self.setMouseTracking(True)
        self.label = QLabel()
        self.v_layout = QVBoxLayout() 
        self.left = left
        self.pt = parent
        self.icon_path = icon_path
        self.move_icon_path = move_icon_path
        self.init_layout()
     
    def init_layout(self):
        self.label.setPixmap(QPixmap(self.icon_path))
        self.v_layout.addWidget(self.label)
        self.setLayout(self.v_layout)
    
    def mouseReleaseEvent(self, ev):
        self.pt.move_func(self.left)
    
    def mouseMoveEvent(self, QMouseEvent):
        self.label.setStyleSheet('background:#dbdbdb;')
        self.label.setPixmap(QPixmap(self.move_icon_path))

    def leaveEvent(self, QMouseEvent):
        self.label.setStyleSheet('background:rgba(0,0,0,0);')
        self.label.setPixmap(QPixmap(self.icon_path))



# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from PyQt5.QtGui import QDesktopServices, QIcon
from PyQt5.QtWidgets import QApplication, QInputDialog, QMainWindow, QAction, QFileDialog, QMessageBox
from PyQt5.QtCore import QMimeData, QUrl
from sentenceLabelLib.MainPanel import MainPanel
import json
from sentenceLabelLib.resources import *
from sentenceLabelLib.utils import Tools
from sentenceLabelLib import utils

class MainWindow(QMainWindow):
    
    def __init__(self) -> None:
        super(MainWindow, self).__init__()
        self.setMinimumSize(1100, 600)
        self.edit_data = None
        self.start = 0
        self.end = 0
        self.middle = 0
        self.is_save = True
        self.file = ''
        self.segmentation = ''
        self.file_menu = self.menuBar().addMenu('File')
        self.edit_menu = self.menuBar().addMenu('Edit')
        self.help_menu = self.menuBar().addMenu('Help')

        self.file_toolbar = self.addToolBar('File')
        self.edit_toolbar = self.addToolBar('Edit')

        self.status_bar = self.statusBar()

        self.open_action = QAction('Open', self)
        self.save_action = QAction('Save', self)
        self.save_as_action = QAction('Save As', self)
        self.close_action = QAction('Close', self)
        
        self.copy_action = QAction('Copy', self)
        self.paste_action = QAction('Paste', self)
        self.left_action = QAction('Left', self)
        self.right_action = QAction('Right', self)
        self.jump_action = QAction('Jump', self)
        self.delete_current_action = QAction('Delete current', self)
        self.delete_all_action = QAction('Delete all', self)

        self.document_action = QAction('Document', self)
        self.main_panel = MainPanel(self, ':/left', ':/right', ':/move-left', ':/move-right')
        self.setWindowIcon(QIcon(':/logo'))
        self.setCentralWidget(self.main_panel)
        self.menu_init()
        self.toolbar_init()
        self.status_bar_init()
        self.action_init()
    
    def menu_init(self):
        self.file_menu.addAction(self.open_action)
        self.file_menu.addAction(self.save_action)
        self.file_menu.addAction(self.save_as_action)
        self.file_menu.addSeparator()
        self.file_menu.addAction(self.close_action)

        self.edit_menu.addAction(self.copy_action)
        self.edit_menu.addAction(self.paste_action)
        self.edit_menu.addSeparator()
        self.edit_menu.addAction(self.left_action)
        self.edit_menu.addAction(self.right_action)
        self.edit_menu.addAction(self.jump_action)
        self.edit_menu.addSeparator()
        self.edit_menu.addAction(self.delete_current_action)
        self.edit_menu.addAction(self.delete_all_action)

        self.help_menu.addAction(self.document_action)

    def toolbar_init(self):
        self.file_toolbar.addAction(self.open_action)
        self.file_toolbar.addAction(self.save_action)
        self.file_toolbar.addAction(self.save_as_action)
        self.file_toolbar.addAction(self.close_action)

        self.edit_toolbar.addAction(self.left_action)
        self.edit_toolbar.addAction(self.right_action)
        self.edit_toolbar.addAction(self.jump_action)
        self.edit_toolbar.addAction(self.delete_current_action)
        self.edit_toolbar.addAction(self.delete_all_action)
        self.edit_toolbar.addAction(self.copy_action)
        self.edit_toolbar.addAction(self.paste_action)
        self.edit_toolbar.addAction(self.document_action)

    def status_bar_init(self):
        self.status_bar.showMessage('Ready Go !!!')
    
    def action_init(self):
        self.open_action.setIcon(QIcon(':/open'))
        self.open_action.setShortcut('Ctrl+O')
        self.open_action.setToolTip('Open an existing file')
        self.open_action.setStatusTip('Open an existing file')
        self.open_action.triggered.connect(self.open_func)

        self.save_action.setIcon(QIcon(':/save'))
        self.save_action.setShortcut('Ctrl+S')
        self.save_action.setToolTip('Save the file')
        self.save_action.setStatusTip('Save the file')
        self.save_action.triggered.connect(self.save_func)

        self.save_as_action.setIcon(QIcon(':/save-as'))
        self.save_as_action.setShortcut('Ctrl+A')
        self.save_as_action.setToolTip('Save the file to a specified location')
        self.save_as_action.setStatusTip('Save the file to a specified location')
        self.save_as_action.triggered.connect(self.save_as_func)

        self.close_action.setIcon(QIcon(':/close'))
        self.close_action.setShortcut('Ctrl+E')
        self.close_action.setToolTip('Close the file')
        self.close_action.setStatusTip('Close the file')
        self.close_action.triggered.connect(self.close_func)


        self.left_action.setIcon(QIcon(':/xiangzuo'))
        self.left_action.setShortcut('A')
        self.left_action.setToolTip('Towards the left')
        self.left_action.setStatusTip('Towards the left')
        self.left_action.triggered.connect(self.left_func)

        self.right_action.setIcon(QIcon(':/xiangyou'))
        self.right_action.setShortcut('D')
        self.right_action.setToolTip('Towards the right')
        self.right_action.setStatusTip('Towards the right')
        self.right_action.triggered.connect(self.right_func)

        self.jump_action.setIcon(QIcon(':/jump'))
        self.jump_action.setShortcut('Ctrl+J')
        self.jump_action.setToolTip('Move to specified article')
        self.jump_action.setStatusTip('Move to specified article')
        self.jump_action.triggered.connect(self.jump_func)

        self.delete_current_action.setIcon(QIcon(':/d1'))
        self.delete_current_action.setToolTip('Delete current labels')
        self.delete_current_action.setStatusTip('Delete current labels')
        self.delete_current_action.triggered.connect(self.delete_current_func)

        self.delete_all_action.setIcon(QIcon(':/d2'))
        self.delete_all_action.setToolTip('Delete all labels')
        self.delete_all_action.setStatusTip('Delete all labels')
        self.delete_all_action.triggered.connect(self.delete_all_func)

        self.copy_action.setIcon(QIcon(':/copy'))
        self.copy_action.setShortcut('Ctrl+C')
        self.copy_action.setToolTip('Copy the text')
        self.copy_action.setStatusTip('Copy the text')
        self.copy_action.triggered.connect(self.copy_func)

        self.paste_action.setIcon(QIcon(':/paste'))
        self.paste_action.setShortcut('Ctrl+V')
        self.paste_action.setToolTip('Paste the text')
        self.paste_action.setStatusTip('Paste the text')
        self.paste_action.triggered.connect(self.paste_func)

        self.document_action.setIcon(QIcon(':/document'))
        self.document_action.setShortcut('Ctrl+D')
        self.document_action.setToolTip('Open the document')
        self.document_action.setStatusTip('Open the document')
        self.document_action.triggered.connect(self.document_func)

        self.mime_data = QMimeData()
        self.clipboard = QApplication.clipboard()

    def delete_current_func(self):
        if self.edit_data:
            self.is_save = False
            self.edit_data[Tools.Data][self.middle][Tools.Sentences] = []
            self.edit_data[Tools.Data][self.middle][Tools.Number] = 0
            self.main_panel.refresh()
    
    def delete_all_func(self):
        if self.edit_data:
            delete = QMessageBox.information(self, 'Delete', 'Is it delete all labels?', QMessageBox.Yes | QMessageBox.No)
            if delete == QMessageBox.Yes:
                self.is_save = False
                for i in range(self.end):
                    self.edit_data[Tools.Data][i][Tools.Sentences] = []
                    self.edit_data[Tools.Data][i][Tools.Number] = 0
                self.main_panel.refresh()

    def closeEvent(self, event):
        save = self.save_file()
        if save == QMessageBox.Yes or save == QMessageBox.No:
            event.accept()
        else:
            event.ignore()

    def save_func(self):
        if not self.file:
            self.save_as_func()
            return
        if not self.edit_data:
            QMessageBox.information(self, 'Save', 'File not open!!!', QMessageBox.Yes)
            return
        with open(self.file, 'w', encoding='utf-8') as f:
            json.dump(self.edit_data, f, indent=4, ensure_ascii=False)
        self.is_save = True

    def save_as_func(self):
        if not self.edit_data:
            QMessageBox.information(self, 'Save', 'File not open!!!', QMessageBox.Yes)
            return
        file = QFileDialog.getSaveFileName(self, 'Save File', './', 'Files (*.json *.txt)')
        if file[0]:
            with open(file[0], 'w', encoding='utf-8') as f:
                json.dump(self.edit_data, f, indent=4, ensure_ascii=False)
            self.file = file[0]
            self.is_save = True

    def close_func(self):
        save = self.save_file()
        if save == QMessageBox.Yes or save == QMessageBox.No:
            self.middle = 0
            self.start = 0
            self.end = 0
            self.edit_data = None
            self.file = ''
            self.is_save = True
            self.segmentation = ''
            self.main_panel.refresh()
            utils.refresh()
        
        return save

    def save_file(self):
        if not self.is_save:
            save = QMessageBox.information(self, 'Save', 'Whether to save changes?', QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
            if save == QMessageBox.Yes:
                with open(self.file, 'w', encoding='utf-8') as f:
                    json.dump(self.edit_data, f, indent=4, ensure_ascii=False)
                self.is_save = True
            return save
        return QMessageBox.Yes

    def left_func(self):
        self.move_func(True)

    def right_func(self):
        self.move_func(False)
    
    def item_is_save(self, d):
        self.is_save = False
        ss = self.edit_data[Tools.Data][self.middle].get(Tools.Sentences, [])
        is_save = True
        if d in ss: 
            ss.remove(d)
        else:
            ss.append(d)
            is_save = False
        self.edit_data[Tools.Data][self.middle][Tools.Sentences] = ss
        self.edit_data[Tools.Data][self.middle][Tools.Number] = len(ss)
        return is_save
        
    def move_func(self, left=False):
        if self.edit_data:
            if left:
                if self.middle - 1 < self.start:
                    QMessageBox.information(self, '', 
                f'''There is no data ahead!!!''', QMessageBox.StandardButton.Ok)
                else:
                    self.middle -= 1
                    self.main_panel.refresh()
            else:
                if self.middle + 1 >= self.end:
                    QMessageBox.information(self, '', 
                f'''There is no data behind!!!''', QMessageBox.StandardButton.Ok)
                else:
                    self.middle += 1
                    self.main_panel.refresh()

    def jump_func(self):
        if not self.edit_data:
            QMessageBox.information(self, 'Jump', 'File not open!!!', QMessageBox.Yes)
            return
        number, ok = QInputDialog.getInt(self, 'Number of articles', 'Please enter the number of articles:')
        if ok:
            if number < self.start:
                QMessageBox.information(self, 'Jump', 'The number is too small!!!', QMessageBox.Yes)
            elif number >= self.end:
                QMessageBox.information(self, 'Jump', 'The number is too large!!!', QMessageBox.Yes)
            else:
                self.middle = number
                self.main_panel.refresh()

    def copy_func(self):
        if not self.edit_data:
            QMessageBox.information(self, 'Copy', 'File not open!!!', QMessageBox.Yes)
            return
        self.mime_data.setText(json.dumps(self.edit_data[Tools.Data][self.middle], indent=4))
        self.clipboard.setMimeData(self.mime_data)
    
    def paste_func(self):
        d = self.clipboard.mimeData().text()
        try:
            jd = json.loads(d)
            utils.check_format(jd)
            if self.edit_data:
                self.edit_data[Tools.Data].install(self.middle, jd)
            else:
                self.edit_data = {
                    Tools.Data: [jd]
                }
                self.segmentation = self.edit_data.get(Tools.Segmentation, utils.SegmentationFunc())
            self.end += 1
            self.is_save = False 
            self.main_panel.refresh()
        except:
            self.data_format_error(is_data=True)


    def document_func(self):
        QDesktopServices.openUrl(QUrl('http://www.myhwx.com/sentence-label'))

    def open_func(self):
        file, _ = QFileDialog.getOpenFileName(self, 'Open File', './', 'Files (*.txt *.json)')
        if file:
            cf = self.close_func()
            if cf == QMessageBox.Cancel:
                return
            try:
                with open(file, 'r', encoding='utf-8') as f:
                    self.edit_data = json.load(f)

                config = self.edit_data.get(Tools.Config, {})

                is_config = utils.injection(config)

                if not is_config:
                    self.config_format_error()
                    return
                try:
                    if Tools.Segmentation in self.edit_data and Tools.SentName in self.edit_data:
                        sn = self.edit_data[Tools.SentName]
                        assert type(sn) == str
                        assert len(sn) >= 1
                        for s_ in sn:
                            assert (s_.lower() <= 'z' and s_.lower() >= 'a') or s_.lower() == '_'
                        Tools.SentList = self.edit_data[Tools.SentName]
                except Exception:
                    QMessageBox.critical(self, 'Variable name Error', f'''<pre>Variable name Error:<p style="color:rgba(255, 0, 0, 0.8)">[sentName] is a variable name</p></pre>''', QMessageBox.StandardButton.Ok)
                    utils.refresh()
                    return
                try:
                    # 动态定义切分文章问方法的函数
                    self.segmentation = self.edit_data.get(Tools.Segmentation, utils.SegmentationFunc())
                    utils.check_segmentation(self.segmentation)
                except Exception:
                    QMessageBox.critical(self, 'Segmentation Error', f'''<pre>Segmentation Error:<p style="color:rgba(255, 0, 0, 0.8)">input: article<br/>output:{Tools.SentList}<br/>{Tools.SentList}: a string list<br/></p></pre>''', QMessageBox.StandardButton.Ok)
                    utils.refresh()
                    self.segmentation = ''
                    return

                for d in self.edit_data[Tools.Data]:
                    utils.check_format(d)
                self.end = len(self.edit_data[Tools.Data])
                self.start = 0
                self.middle = 0
                self.file = file
                self.main_panel.refresh()
            except Exception as ex:
                self.data_format_error()
                utils.refresh()

    def config_format_error(self):
        correct_format = {
            'answer': 'str',
            'question': 'str',
            'article': 'str',
            'options': 'str',
            'data': 'str',
            'sentences': 'str',
            'number': 'str'
            }
        correct_format = json.dumps(correct_format, indent=4)
        QMessageBox.critical(self, 'Format Error', 
        f'''<pre>Correct Format:<pre style="color:rgba(255, 0, 0, 0.8)">{correct_format}</pre></pre>
        ''', QMessageBox.StandardButton.Ok)

    def data_format_error(self, is_data=False):
        correct_format = {
                    Tools.Data: [{Tools.Answer: "str/int", Tools.Options: ["str", "..."], Tools.Question: "str", Tools.Article: "str"}]
                }
        if is_data:
            correct_format = json.dumps(correct_format[Tools.Data][0], indent=4)
        else:
            correct_format = json.dumps(correct_format, indent=4)
        QMessageBox.critical(self, 'Format Error', 
        f'''<pre>Correct Format:<pre style="color:rgba(255, 0, 0, 0.8)">{correct_format}</pre></pre>
        ''', QMessageBox.StandardButton.Ok)

def main():
    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.setWindowTitle('Sentence Label')
    main_window.show()
    return app.exec_()

if __name__ == '__main__':
    sys.exit(main())  


# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class Tools:
    Answer = 'answer'
    
    Question = 'question'
    
    Article = 'article'
    
    Options = 'options'
    
    Data = 'data'

    Sentences = 'sentences'

    Number = 'number'

    Config = 'config'

    Segmentation = 'segmentation'
    
    SentList = 'sent_list'
    
    SentName = 'sentName'

def SegmentationFunc():
    return f'from nltk.tokenize import sent_tokenize\nsent_list = sent_tokenize({Tools.Article})'

def refresh():
    Tools.Answer = 'answer'
    Tools.Question = 'question'
    Tools.Article = 'article'
    Tools.Options = 'options'
    Tools.Data = 'data'
    Tools.Sentences = 'sentences'
    Tools.Number = 'number'
    Tools.SentList = 'sent_list'

def injection(config):
    if type(config) != dict:
        return False
    if type(config) == dict:
        answer = config.get('answer', 'answer')
        question = config.get('question', 'question')
        article = config.get('article', 'article')
        options = config.get('options', 'options')
        data = config.get('data', 'data')
        sentences = config.get('sentences', 'sentences')
        number = config.get('number', 'number')
        if type(answer) != str:
            return False
        if type(question) != str:
            return False
        if type(article) != str:
            return False
        if type(data) != str:
            return False
        if type(options) != str:
            return False
        if type(sentences) != str:
            return False
        if type(number) != str:
            return False
        Tools.Answer = answer
        Tools.Question = question
        Tools.Article = article
        Tools.Options = options
        Tools.Data = data
        Tools.Sentences = sentences
        Tools.Number = number
    return True

def check_segmentation(segmentation):
    exec(f'''{Tools.Article} = "my name is xiaoming. I am very happy every day."\n''' + segmentation)
    sent_list_ = locals()[Tools.SentList]
    type(sent_list_) == list
    for s in sent_list_:
        assert type(s) == str

def check_format(d):
    assert type(d[Tools.Answer]) == str or type(d[Tools.Answer]) == int
    assert d[Tools.Answer] != ''
    if type(d[Tools.Answer]) == str:
        assert len(d[Tools.Answer]) == 1
    if not is_int(d[Tools.Answer]):
        assert is_a2z(d[Tools.Answer])
    assert d[Tools.Question] != ''
    assert type(d[Tools.Question]) == str
    assert d[Tools.Article] != ''
    assert type(d[Tools.Article]) == str
    assert len(d[Tools.Options]) != 0
    for o in d[Tools.Options]:
        assert type(o) == str
        assert o != ''

def is_int(s):
    is_i = True
    try:
        int(s)
    except Exception:
        is_i = False
    return is_i

def is_a2z(s):
    s = s.lower()
    return s >= 'a' and s <= 'z'


# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PyQt5.QtWidgets import QWidget, QSplitter, QHBoxLayout
from PyQt5.QtCore import Qt
from sentenceLabelLib.LeftRightLabel import Label
from sentenceLabelLib.ScrollPanel import ScrollPanel

class MainPanel(QWidget):
    def __init__(self, parent, icon_path_left, icon_path_right, move_icon_path_left, move_icon_path_right):
        super(MainPanel, self).__init__()
        self.left_right = QSplitter(Qt.Orientation.Horizontal, self)
        self.top_bottom = QSplitter(Qt.Orientation.Vertical, self)

        self.left = Label(parent, icon_path_left, move_icon_path_left, True)   
        self.right = Label(parent, icon_path_right, move_icon_path_right)

        self.sp_left = ScrollPanel(parent, ScrollPanel.Left)
        self.sp_top = ScrollPanel(parent, ScrollPanel.Top)
        self.sp_bottom = ScrollPanel(parent, ScrollPanel.Bottom)

        self.main_layout = QHBoxLayout(self)

        self.init_layout()
    
    def init_layout(self):
        
        self.sp_left.setMinimumSize(400, 450)
        self.sp_top.setMinimumSize(500, 300)
        self.sp_bottom.setMinimumSize(500, 150)
        self.left_right.addWidget(self.sp_left)
        self.top_bottom.addWidget(self.sp_top)
        self.top_bottom.addWidget(self.sp_bottom)
        self.left_right.addWidget(self.top_bottom)
        
        self.main_layout.addWidget(self.left)
        self.main_layout.addWidget(self.left_right)
        self.main_layout.addWidget(self.right)
        self.setLayout(self.main_layout)

    def refresh(self):
        self.sp_left.refresh()
        self.sp_top.refresh()
        self.sp_bottom.refresh()



# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from PyQt5.QtGui import QColor, QFont
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QListWidget, QListWidgetItem, QTextBrowser
from sentenceLabelLib.utils import Tools
from sentenceLabelLib import utils
class ScrollPanel(QWidget):
    
    Left = 0
    
    Top = 1

    Bottom = 2

    def __init__(self, parent, panel):
        self.pt = parent
        self.panel = panel
        super(ScrollPanel, self).__init__()
        self.h_layout = QHBoxLayout(self)
        if self.panel == ScrollPanel.Left:
            self.list_widget = QListWidget(self)
        elif self.panel != ScrollPanel.Left:
            self.text_browser = QTextBrowser(self)
        
        self.font_ = QFont()
                    
        self.init_layout()
    
    def init_layout(self):
        
        if self.panel == ScrollPanel.Left:
            self.font_.setFamily('STKaiti')
            self.font_.setPixelSize(22)
            self.h_layout.addWidget(self.list_widget)
            self.list_widget.setWordWrap(True)
            self.list_widget.clicked.connect(lambda: self.list_click(self.list_widget))
            self.list_widget.setStyleSheet('''
                QListWidget::item{
                    padding:5px 0px;
                    font-size: 24px;
                }
            ''')
        elif self.panel != ScrollPanel.Left:
            self.h_layout.addWidget(self.text_browser)

        self.setLayout(self.h_layout)

    def list_click(self, lw):
        item = lw.currentItem()
        d = item.text()
        if self.pt.item_is_save(d):
            item.setForeground(QColor('black'))
        else:
            item.setForeground(QColor('red'))
        
    def refresh(self):
        if self.panel == ScrollPanel.Left:
            self.list_widget.clear()
        else:
            self.text_browser.clear()

        edit_data = self.pt.edit_data
        start = self.pt.start
        end = self.pt.end
        middle = self.pt.middle

        if middle >= start and middle < end and edit_data:
            d = edit_data[Tools.Data][middle]
            if self.panel == ScrollPanel.Left:
                los = {Tools.Article : d[Tools.Article]}
                exec(self.pt.segmentation, {}, los)
                sent_list_ = los[Tools.SentList]
                ss =  self.pt.edit_data[Tools.Data][self.pt.middle].get(Tools.Sentences, [])
                for s in sent_list_:
                    item = QListWidgetItem()
                    item.setText(s)
                    if s in ss:
                        item.setForeground(QColor('red'))
                    item.setFont(self.font_)
                    self.list_widget.addItem(item)

            elif self.panel == ScrollPanel.Top:
                ars = d[Tools.Article].split('\n')
                ar = ''.join([f'''<p style="text-indent: 30px;font-family:STKaiti;">{ar}</p>''' for ar in ars])
                self.text_browser.setText(f'''
                <div style="font-size: 22px;margin:25px;line-height: 28px;">{ar}</div>
                ''')
            else:
                a = d[Tools.Answer]
                if utils.is_int(a):
                    ai = int(a)
                else:
                    ai = ord(a.lower()) - ord('a')
                ops = ''.join([f'<p style="font-family:STKaiti;"><span style="text-decoration:underline;color:red;">{str(i + 1)}. {o}</span></p>' if i == ai else f'<p style="font-family:STKaiti;">{str(i + 1)}. {o}</p>' for i, o in enumerate(d[Tools.Options]) ])
                ls = f'''<div style="font-size: 22px;margin:25px;line-height: 28px;"><div style="font-size: 24px;font-family:STKaiti;">{d[Tools.Question]}</div>{ops}</div>'''
                self.text_browser.setText(ls)



# Copyright 2021 ncdhz

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import os
from sys import platform as _platform

NAME = 'sentenceLabel'
APP = [NAME + '.py']
REQUIRES_PYTHON = '>=3.0.0'
required_packages = find_packages()
required_packages.append('sentenceLabel')
here = os.path.abspath(os.path.dirname(__file__))
about = {}

with open(os.path.join(here, 'sentenceLabelLib', '__init__.py')) as f:
    exec(f.read(), about)

OPTIONS = {
    'argv_emulation': True,
    'iconfile': 'icons/logo.icns'
}

# OS specific settings
SET_REQUIRES = []
if _platform == "linux" or _platform == "linux2":
   # linux
   print('linux')
elif _platform == "darwin":
   # MAC OS X
   SET_REQUIRES.append('py2app')

setup(
    app=APP,
    name=NAME,
    version=about['__version__'],
    author='ncdhz',
    author_email='1137436221@qq.com',
    description='多项选择文章打标软件',
    long_description='`更多信息请关注 Github <https://github.com/ncdhz/sentenceLabel>`',
    url='https://github.com/ncdhz/sentenceLabel',
    python_requires=REQUIRES_PYTHON,
    package_dir={'sentenceLabel': '.'},
    packages=required_packages,
    entry_points={
        'console_scripts': [
            'sentence-label=sentenceLabel.sentenceLabel:main'
        ]
    },
    include_package_data=True,
    install_requires=[
        'pyqt5',
        'nltk'
    ],
    zip_safe=False,
    license="Apache license",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        "Intended Audience :: Science/Research",
        'Intended Audience :: Developers',
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    options={'py2app': OPTIONS},
    setup_requires=SET_REQUIRES,
)
